# Copyright (c) 2020-2024, Vladimir Botka <vbotka@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: lists_mergeby
short_description: Merge two or more lists of dictionaries by a given attribute
version_added: 2.0.0
author: Vladimir Botka (@vbotka)
description:
  - Merge two or more lists by attribute O(index). Optional parameters O(recursive) and O(list_merge) control the merging
    of the nested dictionaries and lists.
  - The function C(merge_hash) from C(ansible.utils.vars) is used.
  - To learn details on how to use the parameters O(recursive) and O(list_merge) see Ansible User's Guide chapter "Using filters
    to manipulate data" section R(Combining hashes/dictionaries, combine_filter) or the filter P(ansible.builtin.combine#filter).
positional: another_list, index
options:
  _input:
    description:
      - A list of dictionaries, or a list of lists of dictionaries.
      - The required type of the C(elements) is set to C(raw) because all elements of O(_input) can be either dictionaries
        or lists.
    type: list
    elements: raw
    required: true
  another_list:
    description:
      - Another list of dictionaries, or a list of lists of dictionaries.
      - This parameter can be specified multiple times.
    type: list
    elements: raw
  index:
    description:
      - The dictionary key that must be present in every dictionary in every list that is used to merge the lists.
    type: string
    required: true
  recursive:
    description:
      - Should the combine recursively merge nested dictionaries (hashes).
      - B(Note:) It does not depend on the value of the C(hash_behaviour) setting in C(ansible.cfg).
    type: boolean
    default: false
  list_merge:
    description:
      - Modifies the behaviour when the dictionaries (hashes) to merge contain arrays/lists.
    type: string
    default: replace
    choices:
      - replace
      - keep
      - append
      - prepend
      - append_rp
      - prepend_rp
"""

EXAMPLES = r"""
# Some results below are manually formatted for better readability. The
# dictionaries' keys will be sorted alphabetically in real output.

- name: Example 1. Merge two lists. The results r1 and r2 are the same.
  ansible.builtin.debug:
    msg: |
      r1: {{ r1 }}
      r2: {{ r2 }}
  vars:
    list1:
      - {index: a, value: 123}
      - {index: b, value: 4}
    list2:
      - {index: a, foo: bar}
      - {index: c, foo: baz}
    r1: "{{ list1 | community.general.lists_mergeby(list2, 'index') }}"
    r2: "{{ [list1, list2] | community.general.lists_mergeby('index') }}"

#  r1:
#    - {index: a, foo: bar, value: 123}
#    - {index: b, value: 4}
#    - {index: c, foo: baz}
#  r2:
#    - {index: a, foo: bar, value: 123}
#    - {index: b, value: 4}
#    - {index: c, foo: baz}

- name: Example 2. Merge three lists
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, value: 123}
      - {index: b, value: 4}
    list2:
      - {index: a, foo: bar}
      - {index: c, foo: baz}
    list3:
      - {index: d, foo: qux}
    r: "{{ [list1, list2, list3] | community.general.lists_mergeby('index') }}"

#  r:
#    - {index: a, foo: bar, value: 123}
#    - {index: b, value: 4}
#    - {index: c, foo: baz}
#    - {index: d, foo: qux}

- name: Example 3. Merge single list. The result is the same as 2.
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, value: 123}
      - {index: b, value: 4}
      - {index: a, foo: bar}
      - {index: c, foo: baz}
      - {index: d, foo: qux}
    r: "{{ [list1, []] | community.general.lists_mergeby('index') }}"

#  r:
#    - {index: a, foo: bar, value: 123}
#    - {index: b, value: 4}
#    - {index: c, foo: baz}
#    - {index: d, foo: qux}

- name: Example 4. Merge two lists. By default, replace nested lists.
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, foo: [X1, X2]}
      - {index: b, foo: [X1, X2]}
    list2:
      - {index: a, foo: [Y1, Y2]}
      - {index: b, foo: [Y1, Y2]}
    r: "{{ [list1, list2] | community.general.lists_mergeby('index') }}"

#  r:
#    - {index: a, foo: [Y1, Y2]}
#    - {index: b, foo: [Y1, Y2]}

- name: Example 5. Merge two lists. Append nested lists.
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, foo: [X1, X2]}
      - {index: b, foo: [X1, X2]}
    list2:
      - {index: a, foo: [Y1, Y2]}
      - {index: b, foo: [Y1, Y2]}
    r: "{{ [list1, list2] | community.general.lists_mergeby('index', list_merge='append') }}"

#  r:
#    - {index: a, foo: [X1, X2, Y1, Y2]}
#    - {index: b, foo: [X1, X2, Y1, Y2]}

- name: Example 6. Merge two lists. By default, do not merge nested dictionaries.
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, foo: {x: 1, y: 2}}
      - {index: b, foo: [X1, X2]}
    list2:
      - {index: a, foo: {y: 3, z: 4}}
      - {index: b, foo: [Y1, Y2]}
    r: "{{ [list1, list2] | community.general.lists_mergeby('index') }}"

#  r:
#    - {index: a, foo: {y: 3, z: 4}}
#    - {index: b, foo: [Y1, Y2]}

- name: Example 7. Merge two lists. Merge nested dictionaries too.
  ansible.builtin.debug:
    var: r
  vars:
    list1:
      - {index: a, foo: {x: 1, y: 2}}
      - {index: b, foo: [X1, X2]}
    list2:
      - {index: a, foo: {y: 3, z: 4}}
      - {index: b, foo: [Y1, Y2]}
    r: "{{ [list1, list2] | community.general.lists_mergeby('index', recursive=true) }}"

#  r:
#    - {index: a, foo: {x:1, y: 3, z: 4}}
#    - {index: b, foo: [Y1, Y2]}
"""

RETURN = r"""
_value:
  description: The merged list.
  type: list
  elements: dictionary
"""

from collections import defaultdict
from collections.abc import Mapping, Sequence
from operator import itemgetter

from ansible.errors import AnsibleFilterError
from ansible.utils.vars import merge_hash


def list_mergeby(x, y, index, recursive=False, list_merge="replace"):
    """Merge 2 lists by attribute 'index'. The function 'merge_hash'
    from ansible.utils.vars is used.  This function is used by the
    function lists_mergeby.
    """

    d = defaultdict(dict)
    for lst in (x, y):
        for elem in lst:
            if not isinstance(elem, Mapping):
                raise AnsibleFilterError(
                    f"Elements of list arguments for lists_mergeby must be dictionaries. {elem} is {type(elem)}"
                )
            if index in elem.keys():
                d[elem[index]].update(merge_hash(d[elem[index]], elem, recursive, list_merge))
    return sorted(d.values(), key=itemgetter(index))


def lists_mergeby(*terms, **kwargs):
    """Merge 2 or more lists by attribute 'index'. To learn details
    on how to use the parameters 'recursive' and 'list_merge' see
    the filter ansible.builtin.combine.
    """

    recursive = kwargs.pop("recursive", False)
    list_merge = kwargs.pop("list_merge", "replace")
    if kwargs:
        raise AnsibleFilterError("'recursive' and 'list_merge' are the only valid keyword arguments.")
    if len(terms) < 2:
        raise AnsibleFilterError("At least one list and index are needed.")

    # allow the user to do `[list1, list2, ...] | lists_mergeby('index')`
    flat_list = []
    for sublist in terms[:-1]:
        if not isinstance(sublist, Sequence):
            raise AnsibleFilterError(
                f"All arguments before the argument index for community.general.lists_mergeby must be lists. {sublist} is {type(sublist)}"
            )
        if len(sublist) > 0:
            if all(isinstance(lst, Sequence) for lst in sublist):
                for item in sublist:
                    flat_list.append(item)
            else:
                flat_list.append(sublist)
    lists = flat_list

    if not lists:
        return []

    if len(lists) == 1:
        return lists[0]

    index = terms[-1]

    if not isinstance(index, str):
        raise AnsibleFilterError(
            f"First argument after the lists for community.general.lists_mergeby must be string. {index} is {type(index)}"
        )

    high_to_low_prio_list_iterator = reversed(lists)
    result = next(high_to_low_prio_list_iterator)
    for list in high_to_low_prio_list_iterator:
        result = list_mergeby(list, result, index, recursive, list_merge)

    return result


class FilterModule:
    """Ansible list filters"""

    def filters(self):
        return {
            "lists_mergeby": lists_mergeby,
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_action_batches
short_description: Resource module for organizations _action _batches
description:
  - Manage operations create, update and delete of the resource organizations _action
    _batches.
  - Create an action batch.
  - Delete an action batch.
  - Update an action batch.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  actionBatchId:
    description: ActionBatchId path parameter. Action batch ID.
    type: str
  actions:
    description: A set of changes to make as part of this action (<a href='https //developer.cisco.com/meraki/api/#...
      details</a>).
    elements: dict
    suboptions:
      body:
        description: The body of the action.
        type: dict
      operation:
        description: The operation to be used.
        type: str
      resource:
        description: Unique identifier for the resource to be acted on.
        type: str
    type: list
  callback:
    description: Details for the callback. Please include either an httpServerId OR
      url and sharedSecret.
    suboptions:
      httpServer:
        description: The webhook receiver used for the callback webhook.
        suboptions:
          id:
            description: The webhook receiver ID that will receive information. If
              specifying this, please leave the url and sharedSecret fields blank.
            type: str
        type: dict
      payloadTemplate:
        description: The payload template of the webhook used for the callback.
        suboptions:
          id:
            description: The ID of the payload template. Defaults to 'wpt_00005' for
              the Callback (included) template.
            type: str
        type: dict
      sharedSecret:
        description: A shared secret that will be included in the requests sent to
          the callback URL. It can be used to verify that the request was sent by
          Meraki. If using this field, please also specify an url.
        type: str
      url:
        description: The callback URL for the webhook target. If using this field,
          please also specify a sharedSecret.
        type: str
    type: dict
  confirmed:
    description: Set to true for immediate execution. Set to false if the action should
      be previewed before executing. This property cannot be unset once it is true.
      Defaults to false.
    type: bool
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  synchronous:
    description: Set to true to force the batch to run synchronous. There can be at
      most 20 actions in synchronous batch. Defaults to false.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationActionBatch
    description: Complete reference of the createOrganizationActionBatch API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-action-batch
  - name: Cisco Meraki documentation for organizations deleteOrganizationActionBatch
    description: Complete reference of the deleteOrganizationActionBatch API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-action-batch
  - name: Cisco Meraki documentation for organizations updateOrganizationActionBatch
    description: Complete reference of the updateOrganizationActionBatch API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-action-batch
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_action_batch,
    organizations.Organizations.delete_organization_action_batch,
    organizations.Organizations.update_organization_action_batch,
  - Paths used are
    post /organizations/{organizationId}/actionBatches,
    delete /organizations/{organizationId}/actionBatches/{actionBatchId},
    put /organizations/{organizationId}/actionBatches/{actionBatchId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    actions:
      - operation: create
        resource: /devices/QXXX-XXXX-XXXX/switch/ports/3
    callback:
      httpServer:
        id: aHR0cHM6Ly93d3cuZXhhbXBsZS5jb20vd2ViaG9va3M=
      payloadTemplate:
        id: wpt_2100
      sharedSecret: secret
      url: https://webhook.site/28efa24e-f830-4d9f-a12b-fbb9e5035031
    confirmed: true
    organizationId: string
    synchronous: true
- name: Delete by id
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    actionBatchId: string
    organizationId: string
- name: Update by id
  cisco.meraki.organizations_action_batches:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    actionBatchId: string
    confirmed: true
    organizationId: string
    synchronous: false
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "organizationId": "string",
      "confirmed": true,
      "synchronous": true,
      "status": {
        "completed": true,
        "failed": true,
        "errors": [
          "string"
        ],
        "createdResources": [
          {
            "id": "string",
            "uri": "string"
          }
        ]
      },
      "actions": [
        {
          "resource": "string",
          "operation": "string",
          "body": {}
        }
      ],
      "callback": {
        "id": "string",
        "url": "string",
        "status": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = r"""
---
module: route53_health_check
version_added: 5.0.0
short_description: Manage health checks in Amazons Route 53 DNS service
description:
  - Creates and deletes DNS Health checks in Amazons Route 53 service.
  - Only the O(port), O(resource_path), O(string_match) and O(request_interval) are
    considered when updating existing health checks.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  state:
    description:
      - Specifies the action to take.
    choices: [ 'present', 'absent' ]
    type: str
    default: 'present'
  disabled:
    description:
      - Stops Route 53 from performing health checks.
      - See the AWS documentation for more details on the exact implications.
        U(https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/health-checks-creating-values.html)
      - Defaults to V(true) when creating a new health check.
    type: bool
    version_added: 2.1.0
    version_added_collection: community.aws
  ip_address:
    description:
      - IP address of the end-point to check. Either this or O(fqdn) has to be provided.
      - IP addresses must be publicly routable.
    type: str
  port:
    description:
      - The port on the endpoint on which you want Amazon Route 53 to perform
        health checks. Required for TCP checks.
    type: int
  type:
    description:
      - The type of health check that you want to create, which indicates how
        Amazon Route 53 determines whether an endpoint is healthy.
      - Once health check is created, type can not be changed.
      - The V(CALCULATED) choice was added in 6.3.0.
    choices: [ 'HTTP', 'HTTPS', 'HTTP_STR_MATCH', 'HTTPS_STR_MATCH', 'TCP', 'CALCULATED' ]
    type: str
  child_health_checks:
    description:
      - The child health checks used for a calculated health check.
      - This parameter takes in the child health checks ids.
    type: list
    elements: str
    version_added: 6.3.0
  health_threshold:
    description:
      - The minimum number of healthy child health checks for a calculated health check to be considered healthy.
    default: 1
    type: int
    version_added: 6.3.0
  resource_path:
    description:
      - The path that you want Amazon Route 53 to request when performing
        health checks. The path can be any value for which your endpoint will
        return an HTTP status code of 2xx or 3xx when the endpoint is healthy,
        for example the file /docs/route53-health-check.html.
      - Mutually exclusive with O(type='TCP').
      - The path must begin with a /
      - Maximum 255 characters.
    type: str
  fqdn:
    description:
      - Domain name of the endpoint to check. Either this or O(ip_address) has
        to be provided. When both are given the O(fqdn) is used in the V(Host:)
        header of the HTTP request.
    type: str
  string_match:
    description:
      - If the check type is HTTP_STR_MATCH or HTTP_STR_MATCH, the string
        that you want Amazon Route 53 to search for in the response body from
        the specified resource. If the string appears in the first 5120 bytes
        of the response body, Amazon Route 53 considers the resource healthy.
    type: str
  request_interval:
    description:
      - The number of seconds between the time that Amazon Route 53 gets a
        response from your endpoint and the time that it sends the next
        health check request.
    default: 30
    choices: [ 10, 30 ]
    type: int
  failure_threshold:
    description:
      - The number of consecutive health checks that an endpoint must pass or
        fail for Amazon Route 53 to change the current status of the endpoint
        from unhealthy to healthy or vice versa.
      - Will default to C(3) if not specified on creation.
    choices: [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ]
    type: int
  health_check_name:
    description:
      - Name of the Health Check.
      - Used together with O(use_unique_names) to set/make use of O(health_check_name) as a unique identifier.
    type: str
    required: false
    aliases: ['name']
    version_added: 4.1.0
    version_added_collection: community.aws
  use_unique_names:
    description:
      - Used together with O(health_check_name) to set/make use of O(health_check_name) as a unique identifier.
    type: bool
    required: false
    version_added: 4.1.0
    version_added_collection: community.aws
  health_check_id:
    description:
      - ID of the health check to be update or deleted.
      - If provided, a health check can be updated or deleted based on the ID as unique identifier.
    type: str
    required: false
    aliases: ['id']
    version_added: 4.1.0
    version_added_collection: community.aws
  measure_latency:
    description:
      - To enable/disable latency graphs to monitor the latency between health checkers in multiple Amazon Web Services regions and your endpoint.
      - Value of O(measure_latency) is immutable and can not be modified after creating a health check.
        See U(https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/monitoring-health-check-latency.html)
    type: bool
    required: false
    version_added: 5.4.0
author:
  - "zimbatm (@zimbatm)"
notes:
  - Support for O(tags) and O(purge_tags) was added in release 2.1.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create a health check for host1.example.com and use it in record
  amazon.aws.route53_health_check:
    state: present
    fqdn: host1.example.com
    type: HTTP_STR_MATCH
    resource_path: /
    string_match: "Hello"
    request_interval: 10
    failure_threshold: 2
  register: my_health_check

- amazon.aws.route53:
    action: create
    zone: "example.com"
    type: CNAME
    record: "www.example.com"
    value: host1.example.com
    ttl: 30
    # Routing policy
    identifier: "host1@www"
    weight: 100
    health_check: "{{ my_health_check.health_check.id }}"

- name: create a simple health check with health_check_name as unique identifier
  amazon.aws.route53_health_check:
    state: present
    health_check_name: ansible
    fqdn: ansible.com
    port: 443
    type: HTTPS
    use_unique_names: true

- name: create a TCP health check with latency graphs enabled
  amazon.aws.route53_health_check:
    state: present
    health_check_name: ansible
    fqdn: ansible.com
    port: 443
    type: HTTPS
    use_unique_names: true
    measure_latency: true

- name: Delete health check
  amazon.aws.route53_health_check:
    state: absent
    fqdn: host1.example.com

- name: Update Health check by ID - update ip_address
  amazon.aws.route53_health_check:
    id: 12345678-abcd-abcd-abcd-0fxxxxxxxxxx
    ip_address: 192.0.2.1

- name: Update Health check by ID - update port
  amazon.aws.route53_health_check:
    id: 12345678-abcd-abcd-abcd-0fxxxxxxxxxx
    ip_address: 8080

- name: Delete Health check by ID
  amazon.aws.route53_health_check:
    state: absent
    id: 12345678-abcd-abcd-abcd-0fxxxxxxxxxx
"""

RETURN = r"""
health_check:
  description: Information about the health check.
  returned: success
  type: dict
  contains:
    action:
      description: The action performed by the module.
      type: str
      returned: When a change is or would be made.
      sample: 'updated'
    id:
      description: The Unique ID assigned by AWS to the health check.
      type: str
      returned: When the health check exists.
      sample: 50ec8a13-9623-4c66-9834-dd8c5aedc9ba
    health_check_version:
      description: The version number of the health check.
      type: int
      returned: When the health check exists.
      sample: 14
    health_check_config:
      description:
        - Detailed information about the health check.
        - May contain additional values from Route 53 health check
          features not yet supported by this module.
      type: dict
      returned: When the health check exists.
      contains:
        type:
          description: The type of the health check.
          type: str
          returned: When the health check exists.
          sample: 'HTTPS_STR_MATCH'
        failure_threshold:
          description:
            - The number of consecutive health checks that an endpoint must pass or fail for Amazon Route 53 to
              change the current status of the endpoint from unhealthy to healthy or vice versa.
          type: int
          returned: When the health check exists.
          sample: 3
        fully_qualified_domain_name:
          description: The FQDN configured for the health check to test.
          type: str
          returned: When the health check exists and an FQDN is configured.
          sample: 'updated'
        ip_address:
          description: The IPv4 or IPv6 IP address of the endpoint to be queried.
          type: str
          returned: When the health check exists and a specific IP address is configured.
          sample: ''
        port:
          description: The port on the endpoint that the health check will query.
          type: str
          returned: When the health check exists.
          sample: 'updated'
        request_interval:
          description: The number of seconds between health check queries.
          type: int
          returned: When the health check exists.
          sample: 30
        resource_path:
          description: The URI path to query when performing an HTTP/HTTPS based health check.
          type: str
          returned: When the health check exists and a resource path has been configured.
          sample: '/healthz'
        search_string:
          description: A string that must be present in the response for a health check to be considered successful.
          type: str
          returned: When the health check exists and a search string has been configured.
          sample: 'ALIVE'
        disabled:
          description: Whether the health check has been disabled or not.
          type: bool
          returned: When the health check exists.
          sample: false
        enable_sni:
          description: This allows the endpoint to respond to HTTPS health check requests with the applicable SSL/TLS certificate.
          type: bool
          returned: When the health check exists.
          sample: false
        inverted:
          description: Specify whether you want Amazon Route 53 to invert the status of a health check.
          type: bool
          returned: When the health check exists.
          sample: false
        measure_latency:
          description:
          - To enable/disable latency graphs to monitor the latency between health checkers in multiple Amazon Web Services regions and your endpoint.
          type: bool
          returned: When the health check exists.
          sample: false
    tags:
      description: A dictionary representing the tags on the health check.
      type: dict
      returned: When the health check exists.
      sample: '{"my_key": "my_value"}'
"""

import typing
import uuid

if typing.TYPE_CHECKING:
    from typing import Any

    from ansible_collections.amazon.aws.plugins.module_utils.botocore import ClientType

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.route53 import AnsibleRoute53Error
from ansible_collections.amazon.aws.plugins.module_utils.route53 import Route53ErrorHandler
from ansible_collections.amazon.aws.plugins.module_utils.route53 import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.route53 import manage_tags


@Route53ErrorHandler.list_error_handler("list health checks")
@AWSRetry.jittered_backoff()
def _list_health_checks(client: ClientType, **params) -> dict[str, Any]:
    return client.list_health_checks(**params)


@Route53ErrorHandler.deletion_error_handler("delete health check")
@AWSRetry.jittered_backoff()
def _delete_health_check(client: ClientType, check_id: str) -> None:
    # This always returns an empty dict, we don't control the API
    client.delete_health_check(
        HealthCheckId=check_id,
    )


@Route53ErrorHandler.common_error_handler("create health check")
@AWSRetry.jittered_backoff()
def _create_health_check(client: ClientType, caller_ref: str, health_check: dict[str, Any]) -> dict[str, Any]:
    return client.create_health_check(
        CallerReference=caller_ref,
        HealthCheckConfig=health_check,
    )


@Route53ErrorHandler.common_error_handler("update health check")
@AWSRetry.jittered_backoff()
def _update_health_check(client: ClientType, check_id: str, **kwargs) -> dict[str, Any]:
    return client.update_health_check(
        HealthCheckId=check_id,
        **kwargs,
    )


@Route53ErrorHandler.list_error_handler("get health check", default_value=None)
@AWSRetry.jittered_backoff()
def _get_health_check(client: ClientType, check_id: str) -> dict[str, Any] | None:
    return client.get_health_check(
        HealthCheckId=check_id,
    )


def iter_health_checks(client: ClientType):
    """Generator that yields all health checks, handling pagination.

    Args:
        client: The Route53 boto3 client

    Yields:
        Health check dictionaries from AWS API

    Note:
        Because Route53 provides no 'filter' mechanism, using a paginator
        would result in (on average) double the number of API calls and
        can get really slow. Additionally, we can't properly wrap the
        paginator, so retrying means starting from scratch with a paginator.
    """
    results = _list_health_checks(client)
    while True:
        yield from results.get("HealthChecks", [])

        if results.get("IsTruncated", False):
            results = _list_health_checks(client, Marker=results.get("NextMarker"))
        else:
            break


def find_health_check(
    client: ClientType,
    ip_addr: str | None,
    fqdn: str | None,
    hc_type: str,
    request_interval: int,
    port: int | None,
) -> dict[str, Any] | None:
    """Searches for health checks that have the exact same set of immutable values.

    Searches across all health checks for one matching the following immutable values:
    - ip_addr
    - fqdn
    - type (immutable)
    - request_interval
    - port

    Args:
        client: The Route53 boto3 client
        ip_addr: IP address to match
        fqdn: Fully qualified domain name to match
        hc_type: Health check type to match
        request_interval: Request interval to match
        port: Port to match

    Returns:
        Health check dict if found, None otherwise
    """
    for check in iter_health_checks(client):
        config = check.get("HealthCheckConfig")
        if (
            config.get("IPAddress", None) == ip_addr
            and config.get("FullyQualifiedDomainName", None) == fqdn
            and config.get("Type") == hc_type
            and config.get("RequestInterval") == request_interval
            and config.get("Port", None) == port
        ):
            return check

    return None


def get_existing_checks_with_name(client: ClientType) -> dict[str, dict[str, Any]]:
    """Get all health checks that have a Name tag.

    Args:
        client: The Route53 boto3 client

    Returns:
        Dictionary mapping health check names to health check dicts
    """
    health_checks_with_name = {}
    for check in iter_health_checks(client):
        tags = describe_health_check(client, check["Id"])["tags"]
        if "Name" in tags:
            health_checks_with_name[tags["Name"]] = check

    return health_checks_with_name


def delete_health_check(client: ClientType, check_id: str | None, check_mode: bool) -> tuple[bool, str | None]:
    """Delete a health check.

    Args:
        client: The Route53 boto3 client
        check_id: The health check ID to delete
        check_mode: Whether running in check mode

    Returns:
        Tuple of (changed, action)
    """
    if not check_id:
        return False, None

    if check_mode:
        return True, "delete"

    _delete_health_check(client, check_id)

    return True, "delete"


def validate_health_check_creation_params(
    healthcheck_type: str,
    string_match: str | None,
    child_health_checks: list[str] | None,
    health_threshold: int | None,
) -> None:
    """Validate that required parameters are present for health check creation.

    Args:
        healthcheck_type: Health check type
        string_match: Search string for STR_MATCH types
        child_health_checks: List of child health check IDs for CALCULATED type
        health_threshold: Minimum healthy children for CALCULATED type

    Raises:
        AnsibleRoute53Error: If required parameters are missing for the given type
    """
    missing_args = []

    if healthcheck_type in ["HTTP_STR_MATCH", "HTTPS_STR_MATCH"] and not string_match:
        missing_args.append("string_match")

    if healthcheck_type == "CALCULATED":
        if not child_health_checks:
            missing_args.append("child_health_checks")
        if not health_threshold:
            missing_args.append("health_threshold")

    if missing_args:
        raise AnsibleRoute53Error(message=f"missing required arguments for creation: {', '.join(missing_args)}")


def build_health_check_config(
    params: dict[str, Any],
    ip_addr: str | None,
    fqdn: str | None,
    healthcheck_type: str,
    request_interval: int,
    port: int | None,
    child_health_checks: list[str] | None,
    health_threshold: int | None,
) -> dict[str, Any]:
    """Build the health check configuration dictionary.

    Args:
        params: Module parameters dict
        ip_addr: IP address to check
        fqdn: Fully qualified domain name to check
        healthcheck_type: Health check type (HTTP, HTTPS, TCP, CALCULATED, etc.)
        request_interval: Request interval in seconds
        port: Port number to check
        child_health_checks: List of child health check IDs (for CALCULATED type)
        health_threshold: Minimum healthy children threshold (for CALCULATED type)

    Returns:
        Health check configuration dictionary
    """
    health_check = {"Type": healthcheck_type}

    if params.get("disabled") is not None:
        health_check["Disabled"] = params.get("disabled")
    if ip_addr:
        health_check["IPAddress"] = ip_addr
    if fqdn:
        health_check["FullyQualifiedDomainName"] = fqdn
    if port:
        health_check["Port"] = port

    if healthcheck_type in ["HTTP", "HTTPS", "HTTP_STR_MATCH", "HTTPS_STR_MATCH"]:
        resource_path = params.get("resource_path")
        if resource_path:
            health_check["ResourcePath"] = resource_path

    if healthcheck_type in ["HTTP_STR_MATCH", "HTTPS_STR_MATCH"]:
        health_check["SearchString"] = params.get("string_match")

    if healthcheck_type == "CALCULATED":
        health_check["ChildHealthChecks"] = child_health_checks
        health_check["HealthThreshold"] = health_threshold
    else:
        health_check["FailureThreshold"] = params.get("failure_threshold") or 3
        health_check["RequestInterval"] = request_interval

    if params.get("measure_latency") is not None:
        health_check["MeasureLatency"] = params.get("measure_latency")

    return health_check


def create_health_check(
    params: dict[str, Any],
    client: ClientType,
    check_mode: bool,
    ip_addr_in: str | None,
    fqdn_in: str | None,
    type_in: str,
    request_interval_in: int,
    port_in: int | None,
    child_health_checks_in: list[str] | None,
    health_threshold_in: int | None,
) -> tuple[bool, str, str | None]:
    """Create a new health check.

    Args:
        params: Module parameters dict
        client: The Route53 boto3 client
        check_mode: Whether running in check mode
        ip_addr_in: IP address to check
        fqdn_in: Fully qualified domain name to check
        type_in: Health check type
        request_interval_in: Request interval in seconds
        port_in: Port number to check
        child_health_checks_in: List of child health check IDs
        health_threshold_in: Minimum healthy children threshold

    Returns:
        Tuple of (changed, action, check_id)
    """
    # In general, if a request is repeated with the same CallerRef it won't
    # result in a duplicate check appearing.  This means we can safely use our
    # retry decorators
    caller_ref = str(uuid.uuid4())

    validate_health_check_creation_params(
        type_in,
        params.get("string_match"),
        child_health_checks_in,
        health_threshold_in,
    )

    health_check = build_health_check_config(
        params,
        ip_addr_in,
        fqdn_in,
        type_in,
        request_interval_in,
        port_in,
        child_health_checks_in,
        health_threshold_in,
    )

    if check_mode:
        return True, "create", None

    result = _create_health_check(client, caller_ref, health_check)

    check_id = result.get("HealthCheck").get("Id")
    return True, "create", check_id


def build_health_check_changes(params: dict[str, Any], existing_check: dict[str, Any]) -> dict[str, Any]:
    """Build dictionary of changes needed to update a health check.

    Compares the desired parameters against the existing health check configuration
    and returns only the fields that need to be updated.

    Updatable fields:
    - ResourcePath
    - SearchString
    - FailureThreshold
    - Disabled
    - IPAddress (only with health_check_id or use_unique_names)
    - Port (only with health_check_id or use_unique_names)
    - FullyQualifiedDomainName (only with health_check_id or use_unique_names)
    - ChildHealthChecks (only with health_check_id or use_unique_names)
    - HealthThreshold (only with health_check_id or use_unique_names)

    Args:
        params: Module parameters dict with desired state
        existing_check: Existing health check data from AWS API

    Returns:
        Dictionary of changes to apply, empty dict if no changes needed
    """
    changes = {}
    existing_config = existing_check.get("HealthCheckConfig", {})

    # Always updatable fields
    resource_path = params.get("resource_path", None)
    if resource_path and resource_path != existing_config.get("ResourcePath"):
        changes["ResourcePath"] = resource_path

    search_string = params.get("string_match", None)
    if search_string and search_string != existing_config.get("SearchString"):
        changes["SearchString"] = search_string

    type_in = params.get("type", None)
    if type_in != "CALCULATED":
        failure_threshold = params.get("failure_threshold", None)
        if failure_threshold and failure_threshold != existing_config.get("FailureThreshold"):
            changes["FailureThreshold"] = failure_threshold

    disabled = params.get("disabled", None)
    if disabled is not None and disabled != existing_config.get("Disabled"):
        changes["Disabled"] = disabled

    # Immutable fields can only be updated when using health_check_id or use_unique_names
    if params.get("health_check_id") or params.get("use_unique_names"):
        ip_address = params.get("ip_address", None)
        if ip_address is not None and ip_address != existing_config.get("IPAddress"):
            changes["IPAddress"] = ip_address

        port = params.get("port", None)
        if port is not None and port != existing_config.get("Port"):
            changes["Port"] = port

        fqdn = params.get("fqdn", None)
        if fqdn is not None and fqdn != existing_config.get("FullyQualifiedDomainName"):
            changes["FullyQualifiedDomainName"] = fqdn

        if type_in == "CALCULATED":
            child_health_checks = params.get("child_health_checks", None)
            if child_health_checks is not None and child_health_checks != existing_config.get("ChildHealthChecks"):
                changes["ChildHealthChecks"] = child_health_checks

            health_threshold = params.get("health_threshold", None)
            if health_threshold is not None and health_threshold != existing_config.get("HealthThreshold"):
                changes["HealthThreshold"] = health_threshold

    return changes


def update_health_check(
    params: dict[str, Any], client: ClientType, check_mode: bool, existing_check: dict[str, Any]
) -> tuple[bool, str | None, str]:
    """Update an existing health check.

    Args:
        params: Module parameters dict
        client: The Route53 boto3 client
        check_mode: Whether running in check mode
        existing_check: Existing health check data from AWS API

    Returns:
        Tuple of (changed, action, check_id)
    """
    check_id = existing_check.get("Id")
    changes = build_health_check_changes(params, existing_check)

    # No changes needed
    if not changes:
        return False, None, check_id

    if check_mode:
        return True, "update", check_id

    # This makes sure we're starting from the version we think we are...
    version_id = existing_check.get("HealthCheckVersion", 1)
    _update_health_check(client, check_id, HealthCheckVersion=version_id, **changes)

    return True, "update", check_id


def describe_health_check(client: ClientType, check_id: str | None) -> dict[str, Any]:
    """Get detailed information about a health check including tags.

    Args:
        client: The Route53 boto3 client
        check_id: Health check ID to describe

    Returns:
        Dictionary containing health check details and tags, or empty dict if not found
    """
    if not check_id:
        return {}

    result = _get_health_check(client, check_id)
    if not result:
        return {}

    health_check = result.get("HealthCheck", {})
    health_check = camel_dict_to_snake_dict(health_check)
    tags = get_tags(client, "healthcheck", check_id)
    health_check["tags"] = tags
    return health_check


def ensure_absent(module, client: ClientType, port_in: int | None) -> tuple[bool, str | None, None]:
    """Handles deletion of a health check.

    Args:
        module: AnsibleAWSModule instance
        client: The Route53 boto3 client
        port_in: Port number (if applicable)

    Returns:
        Tuple of (changed, action, check_id) where check_id is always None for deletions
    """
    check_id_from_param = module.params.get("health_check_id")

    if check_id_from_param:
        result = _get_health_check(client, check_id_from_param)
        if not result:
            module.exit_json(
                changed=False, msg=f"The specified health check with ID: {check_id_from_param} does not exist"
            )

        changed, action = delete_health_check(client, check_id_from_param, module.check_mode)
        return changed, action, None

    existing_check = find_health_check(
        client,
        module.params.get("ip_address"),
        module.params.get("fqdn"),
        module.params.get("type"),
        module.params.get("request_interval"),
        port_in,
    )

    if not existing_check:
        return False, None, None

    check_id = existing_check.get("Id")
    changed, action = delete_health_check(client, check_id, module.check_mode)
    return changed, action, None


def ensure_present(module, client: ClientType, port_in: int | None) -> tuple[bool, str | None, str | None]:
    """Handles creation and updates of a health check.

    Args:
        module: AnsibleAWSModule instance
        client: The Route53 boto3 client
        port_in: Port number (if applicable)

    Returns:
        Tuple of (changed, action, check_id)
    """
    ip_addr_in = module.params.get("ip_address")
    type_in = module.params.get("type")
    fqdn_in = module.params.get("fqdn")
    request_interval_in = module.params.get("request_interval")
    child_health_checks_in = module.params.get("child_health_checks")
    health_threshold_in = module.params.get("health_threshold")
    health_check_name = module.params.get("health_check_name")
    tags = module.params.get("tags")
    use_unique_names = module.params.get("use_unique_names")
    health_check_id_in = module.params.get("health_check_id")

    # Track if user specified tags - needed to determine purge behavior later
    user_specified_tags = tags is not None

    existing_check = None
    if health_check_id_in:
        result = _get_health_check(client, health_check_id_in)
        if not result:
            raise AnsibleRoute53Error(
                message=f"The specified health check with ID: {health_check_id_in} does not exist"
            )
        existing_check = result["HealthCheck"]
    elif not use_unique_names:
        existing_check = find_health_check(
            client,
            module.params.get("ip_address"),
            module.params.get("fqdn"),
            module.params.get("type"),
            module.params.get("request_interval"),
            port_in,
        )

    if use_unique_names:
        existing_checks_with_name = get_existing_checks_with_name(client)
        if tags is None:
            tags = {}
        tags["Name"] = health_check_name

        if health_check_name in existing_checks_with_name:
            changed, action, check_id = update_health_check(
                module.params, client, module.check_mode, existing_checks_with_name[health_check_name]
            )
        else:
            changed, action, check_id = create_health_check(
                module.params,
                client,
                module.check_mode,
                ip_addr_in,
                fqdn_in,
                type_in,
                request_interval_in,
                port_in,
                child_health_checks_in,
                health_threshold_in,
            )
    elif existing_check:
        changed, action, check_id = update_health_check(module.params, client, module.check_mode, existing_check)
    else:
        changed, action, check_id = create_health_check(
            module.params,
            client,
            module.check_mode,
            ip_addr_in,
            fqdn_in,
            type_in,
            request_interval_in,
            port_in,
            child_health_checks_in,
            health_threshold_in,
        )

    if check_id:
        # When use_unique_names is true and user didn't specify tags,
        # we only want to ensure Name tag exists, not purge other tags
        purge_tags = module.params.get("purge_tags") if user_specified_tags else False
        changed |= manage_tags(client, "healthcheck", check_id, tags, purge_tags, module.check_mode)

    return changed, action, check_id


def validate_parameters(params: dict[str, Any]) -> int | None:
    """Validates module parameters and returns the calculated port.

    Args:
        params: Module parameters dict

    Returns:
        Calculated port number, or None if not applicable

    Raises:
        AnsibleRoute53Error: If parameters are invalid
    """
    if not params.get("health_check_id") and not params.get("type"):
        raise AnsibleRoute53Error(
            message="parameter 'type' is required if not updating or deleting health check by ID."
        )

    type_in = params.get("type")
    string_match_in = params.get("string_match")
    port_in = params.get("port")

    # Default port
    if port_in is None:
        if type_in in ["HTTP", "HTTP_STR_MATCH"]:
            port_in = 80
        elif type_in in ["HTTPS", "HTTPS_STR_MATCH"]:
            port_in = 443

    if string_match_in:
        if type_in not in ["HTTP_STR_MATCH", "HTTPS_STR_MATCH"]:
            raise AnsibleRoute53Error(
                message="parameter 'string_match' argument is only for the HTTP(S)_STR_MATCH types"
            )
        if len(string_match_in) > 255:
            raise AnsibleRoute53Error(message="parameter 'string_match' is limited to 255 characters max")

    return port_in


def main():
    argument_spec = dict(
        state=dict(choices=["present", "absent"], default="present"),
        disabled=dict(type="bool"),
        ip_address=dict(),
        port=dict(type="int"),
        type=dict(choices=["HTTP", "HTTPS", "HTTP_STR_MATCH", "HTTPS_STR_MATCH", "TCP", "CALCULATED"]),
        child_health_checks=dict(type="list", elements="str"),
        health_threshold=dict(type="int", default=1),
        resource_path=dict(),
        fqdn=dict(),
        string_match=dict(),
        request_interval=dict(type="int", choices=[10, 30], default=30),
        failure_threshold=dict(type="int", choices=[1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
        health_check_id=dict(type="str", aliases=["id"], required=False),
        health_check_name=dict(type="str", aliases=["name"], required=False),
        use_unique_names=dict(type="bool", required=False),
        measure_latency=dict(type="bool", required=False),
    )

    args_one_of = [
        ["ip_address", "fqdn", "health_check_id", "child_health_checks"],
    ]

    args_if = [
        ["type", "TCP", ("port",)],
        ["type", "CALCULATED", ("child_health_checks", "health_threshold")],
    ]

    args_required_together = [
        ["use_unique_names", "health_check_name"],
    ]

    args_mutually_exclusive = [
        ["health_check_id", "health_check_name"],
        ["child_health_checks", "ip_address"],
        ["child_health_checks", "port"],
        ["child_health_checks", "fqdn"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_one_of=args_one_of,
        required_if=args_if,
        required_together=args_required_together,
        mutually_exclusive=args_mutually_exclusive,
        supports_check_mode=True,
    )

    try:
        port_in = validate_parameters(module.params)

        state_in = module.params.get("state")
        client = module.client("route53", retry_decorator=AWSRetry.jittered_backoff())

        changed = False
        action = None
        check_id = None

        if state_in == "absent":
            changed, action, check_id = ensure_absent(module, client, port_in)
        elif state_in == "present":
            changed, action, check_id = ensure_present(module, client, port_in)

        health_check = describe_health_check(client, check_id)
        health_check["action"] = action
        module.exit_json(
            changed=changed,
            health_check=health_check,
        )
    except AnsibleRoute53Error as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_vrf_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: iosxr_vrf_interfaces
short_description: Resource module to configure VRF interfaces.
description: This module configures and manages the VRF configuration in interface on IOS XR platforms.
version_added: 10.3.0
author: Sagar Paul (@KB-perByte)
notes:
  - Tested against Cisco IOS-XR 7.2.2.
  - This module works with connection C(network_cli).
options:
  config:
    description: A list of VRF interfaces options.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of the interface excluding any logical unit number,
            i.e. GigabitEthernet0/0/0/1.
        type: str
        required: true
      vrf_name:
        description:
          - Vrf that is to be added to the interface.
        type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS
        device by executing the command B(sh running-config interface).
      - The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into Ansible structured data as per the
        resource module's argspec and the value is then returned in the
        I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any
        change on the device.
      - The state I(rendered) will transform the configuration in C(config)
        option to platform specific CLI commands which will be returned in the
        I(rendered) key within the result. For state I(rendered) active
        connection to remote host is not required.
      - The state I(gathered) will fetch the running configuration from device
        and transform it into structured data in the format as per the resource
        module argspec and the value is returned in the I(gathered) key within
        the result.
      - The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into JSON format as per the resource module
        parameters and the value is returned in the I(parsed) key within the
        result. The value of C(running_config) option should be the same format
        as the output of command I(show running-config | include ip route|ipv6
        route) executed on device. For state I(parsed) active connection to
        remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - rendered
      - parsed
    default: merged
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  shutdown
# !

- name: Simple merge selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: merged
    config:
      - name: MgmtEth0/RP0/CPU0/0
      - name: GigabitEthernet0/0/0/0
      - name: GigabitEthernet0/0/0/1
        vrf_name: vrf_C
      - name: GigabitEthernet0/0/0/2
        vrf_name: vrf_D

# Task Output
# -----------
#
# before:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#   - name: GigabitEthernet0/0/0/2
# commands:
#   - interface GigabitEthernet0/0/0/1
#   - vrf vrf_C
#   - interface GigabitEthernet0/0/0/2
#   - vrf vrf_D
# after:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_C
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D

# After state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

# Using replaced

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

- name: Simple replaced selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: replaced
    config:
      - name: MgmtEth0/RP0/CPU0/0
      - name: GigabitEthernet0/0/0/0
      - name: GigabitEthernet0/0/0/1
        vrf_name: vrf_E
      - name: GigabitEthernet0/0/0/2
        vrf_name: vrf_D

# Task Output
# -----------
#
# before:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_C
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D
# commands:
#   - interface GigabitEthernet0/0/0/1
#   - vrf vrf_E
# after:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_E
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D

# After state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_E
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

# Using overridden

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

- name: Simple overridden selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: overridden
    config:
      - name: MgmtEth0/RP0/CPU0/0
      - name: GigabitEthernet0/0/0/0
      - name: GigabitEthernet0/0/0/1
        vrf_name: vrf_E

# Task Output
# -----------
#
# before:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_C
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D
# commands:
#   - interface GigabitEthernet0/0/0/1
#   - vrf vrf_E
#   - interface GigabitEthernet0/0/0/2
#   - no vrf vrf_E
# after:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_E
#   - name: GigabitEthernet0/0/0/2

# After state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_E
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  shutdown
# !

# Using deleted

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_E
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

- name: Simple deleted selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: deleted
    config:
      - name: GigabitEthernet0/0/0/1
        vrf_name: vrf_E

# Task Output
# -----------
#
# before:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_E
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D
# commands:
#   - interface GigabitEthernet0/0/0/1
#   - no vrf vrf_E
# after:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D

# After state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

# Using gathered

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

- name: Simple gathered selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: gathered

# Task Output
# -----------
#
# gathered:
#   - name: MgmtEth0/RP0/CPU0/0
#   - name: GigabitEthernet0/0/0/0
#   - name: GigabitEthernet0/0/0/1
#     vrf_name: vrf_C
#   - name: GigabitEthernet0/0/0/2
#     vrf_name: vrf_D

# Using rendered

# Before state:
# -------------
#
# viosxr#show running-config interfaces
# interface MgmtEth0/RP0/CPU0/0
#  ipv4 address dhcp
# !
# interface GigabitEthernet0/0/0/0
#  description this is interface0
#  cdp
# !
# interface GigabitEthernet0/0/0/1
#  vrf vrf_C
#  shutdown
# !
# interface GigabitEthernet0/0/0/2
#  vrf vrf_D
#  shutdown
# !

- name: Simple rendered selective
  cisco.iosxr.iosxr_vrf_interfaces:
    state: rendered

# Task Output
# -----------
#
# commands:
#   - interface GigabitEthernet0/0/0/1
#   - vrf vrf_C
#   - interface GigabitEthernet0/0/0/2
#   - vrf vrf_D
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - interface GigabitEthernet0/0/0/1
    - no vrf test_vrf1
    - vrf test_vrf2
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - interface GigabitEthernet0/0/0/1
    - no vrf test_vrf1
    - vrf test_vrf2
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.vrf_interfaces.vrf_interfaces import (
    Vrf_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.vrf_interfaces.vrf_interfaces import (
    Vrf_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# Copyright (c) 2014, Red Hat, Inc.
# Copyright (c) 2014, Tim Bielawa <tbielawa@redhat.com>
# Copyright (c) 2014, Magnus Hedemark <mhedemar@redhat.com>
# Copyright (c) 2017, Dag Wieers <dag@wieers.com>
# Copyright (c) 2026, Shreyash Bhosale <shreyashpb16@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    DOCUMENTATION = r"""
options:
  path:
    description:
      - Path to the file to operate on.
      - This file must exist ahead of time.
      - This parameter is required, unless O(xmlstring) is given.
    type: path
    aliases: [dest, file]
  xmlstring:
    description:
      - A string containing XML on which to operate.
      - This parameter is required, unless O(path) is given.
    type: str
  xpath:
    description:
      - A valid XPath expression describing the item(s) you want to operate on.
      - Operates on the document root, V(/), by default.
    type: str
  namespaces:
    description:
      - The namespace C(prefix:uri) mapping for the XPath expression.
      - Needs to be a C(dict), not a C(list) of items.
    type: dict
    default: {}
  strip_cdata_tags:
    description:
      - Remove CDATA tags surrounding text values.
      - Note that this might break your XML file if text values contain characters that could be interpreted as XML.
    type: bool
    default: false
  huge_tree:
    description:
      - Disable libxml2 security restrictions on XML node size or document depth, allowing processing of very large XML files.
      - This option should only be activated when needed, as it disables internal safety limits.
    type: bool
    default: false
requirements:
  - lxml >= 2.3.0
notes:
  - This module does not handle complicated xpath expressions, so limit xpath selectors to simple expressions.
  - Beware that in case your XML elements are namespaced, you need to use the O(namespaces) parameter, see the examples.
  - Namespaces prefix should be used for all children of an element where namespace is defined, unless another namespace is
    defined for them.
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Tim Cragg (@timcragg)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_ip_sla_monitoring_policy
short_description: Manage IP SLA Monitoring Policies (fv:IPSLAMonitoringPol)
description:
- Manage Internet Protocol (IP) Service Level Agreement (SLA) Monitoring Policies used for L4-L7 Policy Based Redirection
options:
  tenant:
    description:
    - The name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
  name:
    description:
    - The SLA Policy name.
    type: str
    aliases: [ sla_policy ]
  description:
    description:
    - The SLA Policy description.
    type: str
  sla_type:
    description:
    - The type of monitoring.
    - The APIC defaults to C(icmp) when unset during creation.
    - I(sla_type=http) is only supported in APIC v5.0 and above.
    type: str
    choices: [ icmp, tcp, l2ping, http ]
  sla_port:
    description:
    - The Port to monitor for TCP SLAs.
    - Permitted values are in the range of [1, 65535].
    type: int
    aliases: [ destination_port ]
  frequency:
    description:
    - Defines the interval at which probes are conducted.
    - The APIC defaults to C(60) when unset during creation.
    - Permitted values are in the range of [1, 300].
    type: int
  multiplier:
    description:
    - Specifies the number of consecutive probe failures required to determine the SLA as down.
    - The APIC defaults to C(3) when unset during creation.
    - Permitted values are in the range of [1, 100].
    type: int
    aliases: [ detect_multiplier ]
  request_data_size:
    description:
    - The number of bytes to send in the request.
    - Only used if I(sla_type=http).
    - The APIC defaults to C(28) when unset during creation.
    - Permitted values are in the range of [0, 17512]
    - This is only supported in APIC v5.1 and above.
    type: int
  type_of_service:
    description:
    - The Type of Service (ToS) value to set in the IPv4 header.
    - The APIC defaults to C(0) when unset during creation.
    - Permitted values are in the range of [0, 255].
    - This is only supported in APIC v5.1 and above.
    type: int
    aliases: [ tos ]
  operation_timeout:
    description:
    - The amount of time in milliseconds that the IP SLA operation waits for a response from its request packet.
    - The APIC defaults to C(900) when unset during creation.
    - Permitted values are in the range of [0, 604800000].
    - This is only supported in APIC v5.1 and above.
    type: int
  threshold:
    description:
    - The upper threshold value in milliseconds for calculating network monitoring statistics created by the IP SLA operation.
    - The value specified for this property must not exceed the value specified for I(operation_timeout).
    - The APIC defaults to C(900) when unset during creation.
    - Permitted values are in the range of [0, 604800000].
    - This is only supported in APIC v5.1 and above.
    type: int
  traffic_class:
    description:
    - Sets the Traffic Class value in the IPv6 header.
    - The APIC defaults to C(0) when unset during creation.
    - Permitted values are in the range of [0, 255].
    - This is only supported in APIC v5.1 and above.
    type: int
    aliases: ["traffic_class_value"]
  http_version:
    description:
    - The HTTP version to use.
    - The APIC defaults to C(1.0) when unset during creation.
    - This is only supported in APIC v5.0 and above.
    type: str
    choices: [ "1.0", "1.1" ]
  http_uri:
    description:
    - The HTTP URI to use as the SLA destination.
    - The APIC defaults to C(/) when unset during creation.
    - This is only supported in APIC v5.0 and above.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The I(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) modules can be used for this.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:IPSLAMonitoringPol)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
"""

EXAMPLES = r"""
- name: Add a new ICMP SLA monitoring policy
  cisco.aci.aci_ip_sla_monitoring_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    name: my_policy
    sla_type: icmp
    frequency: 40
    multiplier: 6
    state: present
  delegate_to: localhost

- name: Update to TCP SLA monitoring policy
  cisco.aci.aci_ip_sla_monitoring_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    name: my_policy
    sla_type: tcp
    sla_port: 2345
    frequency: 45
    multiplier: 5
    state: present
  delegate_to: localhost

- name: Query an SLA monitoring policy
  cisco.aci.aci_ip_sla_monitoring_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    name: my_policy
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all SLA monitoring policies
  cisco.aci.aci_ip_sla_monitoring_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete an SLA monitoring policy
  cisco.aci.aci_ip_sla_monitoring_policy:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    name: my_policy
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible_collections.cisco.aci.plugins.module_utils.constants import HTTP_VERSIONS_MAPPING


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name=dict(type="str", aliases=["sla_policy"]),
        description=dict(type="str"),
        sla_type=dict(type="str", choices=["icmp", "tcp", "l2ping", "http"]),
        sla_port=dict(type="int", aliases=["destination_port"]),
        frequency=dict(type="int"),
        multiplier=dict(type="int", aliases=["detect_multiplier"]),
        request_data_size=dict(type="int"),
        type_of_service=dict(type="int", aliases=["tos"]),
        operation_timeout=dict(type="int"),
        threshold=dict(type="int"),
        traffic_class=dict(type="int", aliases=["traffic_class_value"]),
        http_version=dict(type="str", choices=list(HTTP_VERSIONS_MAPPING)),
        http_uri=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "name"]],
            ["state", "present", ["tenant", "name"]],
        ],
    )

    tenant = module.params.get("tenant")
    state = module.params.get("state")
    description = module.params.get("description")
    name = module.params.get("name")
    sla_type = module.params.get("sla_type")
    sla_port = module.params.get("sla_port")
    frequency = module.params.get("frequency")
    multiplier = module.params.get("multiplier")
    request_data_size = module.params.get("request_data_size")
    type_of_service = module.params.get("type_of_service")
    operation_timeout = module.params.get("operation_timeout")
    threshold = module.params.get("threshold")
    traffic_class = module.params.get("traffic_class")
    http_version = HTTP_VERSIONS_MAPPING.get(module.params.get("http_version"))
    http_uri = module.params.get("http_uri")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvIPSLAMonitoringPol",
            aci_rn="ipslaMonitoringPol-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        ),
    )
    aci.get_existing()

    if state == "present":
        if sla_port is not None and sla_type != "tcp":
            aci.fail_json("Setting 'sla_port' is not allowed when 'sla_type' is not set to 'tcp'.")
        if sla_type == "tcp" and request_data_size is not None:
            aci.fail_json("Setting 'request_data_size' is not allowed when 'sla_type' is set to 'tcp'.")

        if sla_type == "http":
            sla_port = 80
        elif sla_type != "tcp":
            sla_port = 0

        aci.payload(
            aci_class="fvIPSLAMonitoringPol",
            class_config=dict(
                name=name,
                descr=description,
                slaType=sla_type,
                slaPort=sla_port,
                slaFrequency=frequency,
                slaDetectMultiplier=multiplier,
                reqDataSize=request_data_size,
                ipv4Tos=type_of_service,
                timeout=operation_timeout,
                threshold=threshold,
                ipv6TrfClass=traffic_class,
                httpVersion=http_version,
                httpUri=http_uri,
            ),
        )
        aci.get_diff(aci_class="fvIPSLAMonitoringPol")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

REQUESTS_IMPORT_ERROR: str | None  # pylint: disable=invalid-name
try:
    from requests import Session  # noqa: F401, pylint: disable=unused-import
    from requests.adapters import (  # noqa: F401, pylint: disable=unused-import
        HTTPAdapter,
    )
    from requests.exceptions import (  # noqa: F401, pylint: disable=unused-import
        HTTPError,
        InvalidSchema,
    )
except ImportError:
    REQUESTS_IMPORT_ERROR = traceback.format_exc()  # pylint: disable=invalid-name

    class Session:  # type: ignore
        __attrs__: list[t.Never] = []

    class HTTPAdapter:  # type: ignore
        __attrs__: list[t.Never] = []

    class HTTPError(Exception):  # type: ignore
        pass

    class InvalidSchema(Exception):  # type: ignore
        pass

else:
    REQUESTS_IMPORT_ERROR = None  # pylint: disable=invalid-name


URLLIB3_IMPORT_ERROR: str | None = None  # pylint: disable=invalid-name
try:
    from requests.packages import urllib3  # pylint: disable=unused-import

    from requests.packages.urllib3 import (  # type: ignore  # pylint: disable=unused-import  # isort: skip
        connection as urllib3_connection,
    )
except ImportError:
    try:
        import urllib3  # pylint: disable=unused-import
        from urllib3 import (
            connection as urllib3_connection,  # pylint: disable=unused-import
        )
    except ImportError:
        URLLIB3_IMPORT_ERROR = traceback.format_exc()  # pylint: disable=invalid-name

        class _HTTPConnectionPool:
            pass

        class _HTTPConnection:
            pass

        class FakeURLLIB3:
            def __init__(self) -> None:
                self._collections = self
                self.poolmanager = self
                self.connection = self
                self.connectionpool = self

                self.RecentlyUsedContainer = object()  # pylint: disable=invalid-name
                self.PoolManager = object()  # pylint: disable=invalid-name
                self.match_hostname = object()
                self.HTTPConnectionPool = (  # pylint: disable=invalid-name
                    _HTTPConnectionPool
                )

        class FakeURLLIB3Connection:
            def __init__(self) -> None:
                self.HTTPConnection = _HTTPConnection  # pylint: disable=invalid-name

        urllib3 = FakeURLLIB3()
        urllib3_connection = FakeURLLIB3Connection()


def fail_on_missing_imports() -> None:
    if REQUESTS_IMPORT_ERROR is not None:
        from .errors import MissingRequirementException  # pylint: disable=cyclic-import

        raise MissingRequirementException(
            "You have to install requests", "requests", REQUESTS_IMPORT_ERROR
        )
    if URLLIB3_IMPORT_ERROR is not None:
        from .errors import MissingRequirementException  # pylint: disable=cyclic-import

        raise MissingRequirementException(
            "You have to install urllib3", "urllib3", URLLIB3_IMPORT_ERROR
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_vhba_template
short_description: Manage vHBA Templates for Cisco Intersight
description:
  - Create, update, and delete vHBA Templates on Cisco Intersight.
  - vHBA Templates define fibre channel interface configurations that can be used by SAN Connectivity policies.
  - Templates provide a standardized way to configure vHBAs with consistent fibre channel policies.
  - vHBA Templates are only applicable for FI-Attached (Fabric Interconnect) deployments.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/VhbaTemplate/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the vHBA Template.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the vHBA Template.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enable_override:
    description:
      - When enabled, the configuration of the derived instances may override the template configuration.
    type: bool
    default: false
  vhba_type:
    description:
      - vHBA Type configuration for the template.
      - This configuration is supported only on Cisco VIC 14XX series and higher series of adapters.
      - Required when state is present.
    type: str
    choices: ['fc-initiator', 'fc-nvme-initiator', 'fc-nvme-target', 'fc-target']
  switch_id:
    description:
      - The fabric port to which the vHBA will be associated.
    type: str
    choices: ['A', 'B']
    default: 'A'
  persistent_bindings:
    description:
      - Enables retention of LUN ID associations in memory until they are manually cleared.
    type: bool
    default: false
  wwpn_pool_name:
    description:
      - The WWPN pool that is assigned to the vHBA Template.
      - Required when state is present.
    type: str
  fibre_channel_network_policy_name:
    description:
      - Name of the Fibre Channel Network Policy.
      - Required when state is present.
    type: str
  fibre_channel_qos_policy_name:
    description:
      - Name of the Fibre Channel QoS Policy.
      - Required when state is present.
    type: str
  fibre_channel_adapter_policy_name:
    description:
      - Name of the Fibre Channel Adapter Policy.
      - Required when state is present.
    type: str
  fibre_channel_zone_policy_names:
    description:
      - List of Fibre Channel Zone Policy names.
      - Relationship to the FC Zone policies to configure Zones on the switch.
    type: list
    elements: str
  pin_group_name:
    description:
      - Pingroup name associated to vHBA for static pinning.
      - SCP deploy will resolve pingroup name and fetches the corresponding uplink port/port channel to pin the vHBA traffic.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic vHBA Template with WWPN pool
  cisco.intersight.intersight_vhba_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "basic-vhba-template"
    description: "Basic vHBA template for production servers"
    enable_override: false
    vhba_type: "fc-initiator"
    switch_id: "A"
    persistent_bindings: false
    wwpn_pool_name: "default-wwpn-pool"
    fibre_channel_network_policy_name: "fc-network-policy"
    fibre_channel_qos_policy_name: "fc-qos-policy"
    fibre_channel_adapter_policy_name: "fc-adapter-policy"
    tags:
      - Key: "Environment"
        Value: "Production"
      - Key: "Site"
        Value: "DataCenter-A"
    state: present

- name: Create a vHBA Template with override enabled
  cisco.intersight.intersight_vhba_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "override-vhba-template"
    description: "vHBA template with override enabled"
    enable_override: true
    vhba_type: "fc-target"
    switch_id: "B"
    persistent_bindings: true
    wwpn_pool_name: "default-wwpn-pool"
    fibre_channel_network_policy_name: "fc-network-policy"
    fibre_channel_qos_policy_name: "fc-qos-policy"
    fibre_channel_adapter_policy_name: "fc-adapter-policy"
    state: present

- name: Create a vHBA Template with FC Zone policies
  cisco.intersight.intersight_vhba_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "vhba-with-zones"
    description: "vHBA template with FC zone policies"
    enable_override: false
    vhba_type: "fc-initiator"
    switch_id: "A"
    persistent_bindings: false
    wwpn_pool_name: "default-wwpn-pool"
    fibre_channel_network_policy_name: "fc-network-policy"
    fibre_channel_qos_policy_name: "fc-qos-policy"
    fibre_channel_adapter_policy_name: "fc-adapter-policy"
    fibre_channel_zone_policy_names:
      - "fc-zone-policy-1"
      - "fc-zone-policy-2"
    state: present

- name: Create a vHBA Template with pin group
  cisco.intersight.intersight_vhba_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "vhba-pinned-template"
    description: "vHBA template with static pinning"
    enable_override: false
    vhba_type: "fc-nvme-initiator"
    switch_id: "A"
    persistent_bindings: true
    wwpn_address_type: "pool"
    wwpn_pool_name: "nvme-wwpn-pool"
    fibre_channel_network_policy_name: "fc-network-policy"
    fibre_channel_qos_policy_name: "fc-qos-policy"
    fibre_channel_adapter_policy_name: "fc-adapter-policy"
    pin_group_name: "pingroup-a"
    state: present

- name: Delete a vHBA Template
  cisco.intersight.intersight_vhba_template:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-vhba-template"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "test-vhba-template",
        "ObjectType": "vnic.VhbaTemplate",
        "EnableOverride": false,
        "Type": "fc-initiator",
        "SwitchId": "A",
        "PersistentBindings": false,
        "WwpnAddressType": "POOL",
        "WwpnPool": {
            "Name": "default-wwpn-pool",
            "ObjectType": "fcpool.Pool"
        },
        "FcNetworkPolicy": {
            "Name": "fc-network-policy",
            "ObjectType": "vnic.FcNetworkPolicy"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec
from ansible_collections.cisco.intersight.plugins.module_utils.intersight_vhba_utils import (
    get_policy_moid_with_org, resolve_policy_moids_from_mappings, resolve_fc_zone_policies,
    get_vhba_template_policy_mappings
)


def validate_input(module):
    """
    Validate module input parameters.
    """
    if module.params['state'] == 'present':
        # Validate required fields for vHBA Template creation
        required_fields = [
            'vhba_type',
            'wwpn_pool_name',
            'fibre_channel_network_policy_name',
            'fibre_channel_qos_policy_name',
            'fibre_channel_adapter_policy_name'
        ]
        for field in required_fields:
            if not module.params.get(field):
                module.fail_json(msg=f"{field} is required when state is 'present'")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable_override=dict(type='bool', default=False),
        vhba_type=dict(type='str', choices=['fc-initiator', 'fc-nvme-initiator', 'fc-nvme-target', 'fc-target']),
        switch_id=dict(type='str', choices=['A', 'B'], default='A'),
        persistent_bindings=dict(type='bool', default=False),
        wwpn_pool_name=dict(type='str'),
        fibre_channel_network_policy_name=dict(type='str'),
        fibre_channel_qos_policy_name=dict(type='str'),
        fibre_channel_adapter_policy_name=dict(type='str'),
        fibre_channel_zone_policy_names=dict(type='list', elements='str'),
        pin_group_name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )
    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure vHBA Template
    resource_path = '/vnic/VhbaTemplates'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Add vHBA Template specific parameters
        intersight.api_body['EnableOverride'] = intersight.module.params['enable_override']
        intersight.api_body['Type'] = intersight.module.params['vhba_type']
        intersight.api_body['SwitchId'] = intersight.module.params['switch_id']
        intersight.api_body['PersistentBindings'] = intersight.module.params['persistent_bindings']

        # Cache for policy MOIDs to avoid redundant API calls
        policy_cache = {}
        organization_name = intersight.module.params['organization']

        # Resolve WWPN Pool MOID
        wwpn_pool_moid = get_policy_moid_with_org(
            intersight, policy_cache, module, '/fcpool/Pools',
            intersight.module.params['wwpn_pool_name'], organization_name, 'WWPN Pool'
        )
        intersight.api_body['WwpnPool'] = wwpn_pool_moid

        # Resolve FC policy MOIDs
        policy_mappings = get_vhba_template_policy_mappings()
        policy_moids = resolve_policy_moids_from_mappings(intersight, policy_cache, module, intersight.module.params, policy_mappings, organization_name)
        intersight.api_body.update(policy_moids)

        # Resolve Fibre Channel Zone Policies (optional)
        fc_zone_policy_names = intersight.module.params.get('fibre_channel_zone_policy_names')
        fc_zone_policy_moids = resolve_fc_zone_policies(intersight, policy_cache, module, fc_zone_policy_names, organization_name)
        intersight.api_body['FcZonePolicies'] = fc_zone_policy_moids

        # Add pin group name if specified
        if intersight.module.params.get('pin_group_name'):
            intersight.api_body['PinGroupName'] = intersight.module.params['pin_group_name']

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Based on local.py (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
# and chroot.py     (c) 2013, Maykel Moya <mmoya@speedyrails.com>
# and jail.py       (c) 2013, Michael Scherer <misc@zarb.org>
# (c) 2015, Dagobert Michelsen <dam@baltic-online.de>
# (c) 2015, Toshio Kuratomi <tkuratomi@ansible.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Ansible Core Team
name: zone
short_description: Run tasks in a zone instance
description:
  - Run commands or put/fetch files to an existing zone.
options:
  remote_addr:
    description:
      - Zone identifier.
    type: string
    default: inventory_hostname
    vars:
      - name: ansible_host
      - name: ansible_zone_host
"""

import os
import os.path
import subprocess
import traceback
from shlex import quote as shlex_quote

from ansible.errors import AnsibleError
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_bytes
from ansible.plugins.connection import BUFSIZE, ConnectionBase
from ansible.utils.display import Display

display = Display()


class Connection(ConnectionBase):
    """Local zone based connections"""

    transport = "community.general.zone"
    has_pipelining = True
    has_tty = False

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super().__init__(play_context, new_stdin, *args, **kwargs)

        self.zone = self._play_context.remote_addr

        if os.geteuid() != 0:
            raise AnsibleError("zone connection requires running as root")

        self.zoneadm_cmd = to_bytes(self._search_executable("zoneadm"))
        self.zlogin_cmd = to_bytes(self._search_executable("zlogin"))

        if self.zone not in self.list_zones():
            raise AnsibleError(f"incorrect zone name {self.zone}")

    @staticmethod
    def _search_executable(executable):
        try:
            return get_bin_path(executable)
        except ValueError as e:
            raise AnsibleError(f"{executable} command not found in PATH") from e

    def list_zones(self):
        process = subprocess.Popen(
            [self.zoneadm_cmd, "list", "-ip"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )

        zones = []
        for line in process.stdout.readlines():
            # 1:work:running:/zones/work:3126dc59-9a07-4829-cde9-a816e4c5040e:native:shared
            s = line.split(":")
            if s[1] != "global":
                zones.append(s[1])

        return zones

    def get_zone_path(self):
        # solaris10vm# zoneadm -z cswbuild list -p
        # -:cswbuild:installed:/zones/cswbuild:479f3c4b-d0c6-e97b-cd04-fd58f2c0238e:native:shared
        process = subprocess.Popen(
            [self.zoneadm_cmd, "-z", to_bytes(self.zone), "list", "-p"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        # stdout, stderr = p.communicate()
        path = process.stdout.readlines()[0].split(":")[3]
        return f"{path}/root"

    def _connect(self):
        """connect to the zone; nothing to do here"""
        super()._connect()
        if not self._connected:
            display.vvv("THIS IS A LOCAL ZONE DIR", host=self.zone)
            self._connected = True

    def _buffered_exec_command(self, cmd, stdin=subprocess.PIPE):
        """run a command on the zone.  This is only needed for implementing
        put_file() get_file() so that we don't have to read the whole file
        into memory.

        compared to exec_command() it looses some niceties like being able to
        return the process's exit code immediately.
        """
        # NOTE: zlogin invokes a shell (just like ssh does) so we do not pass
        # this through /bin/sh -c here.  Instead it goes through the shell
        # that zlogin selects.
        local_cmd = [self.zlogin_cmd, self.zone, cmd]
        local_cmd = map(to_bytes, local_cmd)

        display.vvv(f"EXEC {local_cmd}", host=self.zone)
        p = subprocess.Popen(local_cmd, shell=False, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        return p

    def exec_command(self, cmd, in_data=None, sudoable=False):
        """run a command on the zone"""
        super().exec_command(cmd, in_data=in_data, sudoable=sudoable)

        p = self._buffered_exec_command(cmd)

        stdout, stderr = p.communicate(in_data)
        return p.returncode, stdout, stderr

    def _prefix_login_path(self, remote_path):
        """Make sure that we put files into a standard path

        If a path is relative, then we need to choose where to put it.
        ssh chooses $HOME but we aren't guaranteed that a home dir will
        exist in any given chroot.  So for now we're choosing "/" instead.
        This also happens to be the former default.

        Can revisit using $HOME instead if it is a problem
        """
        if not remote_path.startswith(os.path.sep):
            remote_path = os.path.join(os.path.sep, remote_path)
        return os.path.normpath(remote_path)

    def put_file(self, in_path, out_path):
        """transfer a file from local to zone"""
        super().put_file(in_path, out_path)
        display.vvv(f"PUT {in_path} TO {out_path}", host=self.zone)

        out_path = shlex_quote(self._prefix_login_path(out_path))
        try:
            with open(in_path, "rb") as in_file:
                if not os.fstat(in_file.fileno()).st_size:
                    count = " count=0"
                else:
                    count = ""
                try:
                    p = self._buffered_exec_command(f"dd of={out_path} bs={BUFSIZE}{count}", stdin=in_file)
                except OSError as e:
                    raise AnsibleError("jail connection requires dd command in the jail") from e
                try:
                    stdout, stderr = p.communicate()
                except Exception as e:
                    traceback.print_exc()
                    raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
                if p.returncode != 0:
                    raise AnsibleError(f"failed to transfer file {in_path} to {out_path}:\n{stdout}\n{stderr}")
        except OSError as e:
            raise AnsibleError(f"file or module does not exist at: {in_path}") from e

    def fetch_file(self, in_path, out_path):
        """fetch a file from zone to local"""
        super().fetch_file(in_path, out_path)
        display.vvv(f"FETCH {in_path} TO {out_path}", host=self.zone)

        in_path = shlex_quote(self._prefix_login_path(in_path))
        try:
            p = self._buffered_exec_command(f"dd if={in_path} bs={BUFSIZE}")
        except OSError as e:
            raise AnsibleError("zone connection requires dd command in the zone") from e

        with open(out_path, "wb+") as out_file:
            try:
                chunk = p.stdout.read(BUFSIZE)
                while chunk:
                    out_file.write(chunk)
                    chunk = p.stdout.read(BUFSIZE)
            except Exception as e:
                traceback.print_exc()
                raise AnsibleError(f"failed to transfer file {in_path} to {out_path}") from e
            stdout, stderr = p.communicate()
            if p.returncode != 0:
                raise AnsibleError(f"failed to transfer file {in_path} to {out_path}:\n{stdout}\n{stderr}")

    def close(self):
        """terminate the connection; nothing to do here"""
        super().close()
        self._connected = False



#!/usr/bin/python

# Copyright (c) 2017, Alejandro Gomez <alexgomez2202@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gunicorn
short_description: Run gunicorn with various settings
description:
  - Starts gunicorn with the parameters specified. Common settings for gunicorn configuration are supported. For additional
    configuration use a config file See U(https://gunicorn-docs.readthedocs.io/en/latest/settings.html) for more options.
    It's recommended to always use the chdir option to avoid problems with the location of the app.
requirements: [gunicorn]
author:
  - "Alejandro Gomez (@agmezr)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  app:
    type: str
    required: true
    aliases: ['name']
    description:
      - The app module. A name refers to a WSGI callable that should be found in the specified module.
  venv:
    type: path
    aliases: ['virtualenv']
    description:
      - Path to the virtualenv directory.
  config:
    type: path
    description:
      - Path to the gunicorn configuration file.
    aliases: ['conf']
  chdir:
    type: path
    description:
      - Chdir to specified directory before apps loading.
  pid:
    type: path
    description:
      - A filename to use for the PID file. If not set and not found on the configuration file a tmp pid file is created to
        check a successful run of gunicorn.
  worker:
    type: str
    choices: ['sync', 'eventlet', 'gevent', 'tornado ', 'gthread', 'gaiohttp']
    description:
      - The type of workers to use. The default class (sync) should handle most "normal" types of workloads.
  user:
    type: str
    description:
      - Switch worker processes to run as this user.
notes:
  - If not specified on config file, a temporary error log is created on C(/tmp) directory. Please make sure you have write
    access in C(/tmp) directory. Not needed but it is helpful to identify any problem with configuration.
"""

EXAMPLES = r"""
- name: Simple gunicorn run example
  community.general.gunicorn:
    app: 'wsgi'
    chdir: '/workspace/example'

- name: Run gunicorn on a virtualenv
  community.general.gunicorn:
    app: 'wsgi'
    chdir: '/workspace/example'
    venv: '/workspace/example/venv'

- name: Run gunicorn with a config file
  community.general.gunicorn:
    app: 'wsgi'
    chdir: '/workspace/example'
    conf: '/workspace/example/gunicorn.cfg'

- name: Run gunicorn as ansible user with specified pid and config file
  community.general.gunicorn:
    app: 'wsgi'
    chdir: '/workspace/example'
    conf: '/workspace/example/gunicorn.cfg'
    venv: '/workspace/example/venv'
    pid: '/workspace/example/gunicorn.pid'
    user: 'ansible'
"""

RETURN = r"""
gunicorn:
  description: Process ID of gunicorn.
  returned: changed
  type: str
  sample: "1234"
"""

import os
import time

from ansible.module_utils.basic import AnsibleModule


def search_existing_config(config, option):
    """search in config file for specified option"""
    if config and os.path.isfile(config):
        with open(config) as f:
            for line in f:
                if option in line:
                    return line
    return None


def remove_tmp_file(file_path):
    """remove temporary files"""
    if os.path.isfile(file_path):
        os.remove(file_path)


def main():
    # available gunicorn options on module
    gunicorn_options = {
        "config": "-c",
        "chdir": "--chdir",
        "worker": "-k",
        "user": "-u",
    }

    module = AnsibleModule(
        argument_spec=dict(
            app=dict(required=True, type="str", aliases=["name"]),
            venv=dict(type="path", aliases=["virtualenv"]),
            config=dict(type="path", aliases=["conf"]),
            chdir=dict(type="path"),
            pid=dict(type="path"),
            user=dict(type="str"),
            worker=dict(type="str", choices=["sync", "eventlet", "gevent", "tornado ", "gthread", "gaiohttp"]),
        )
    )

    # temporary files in case no option provided
    tmp_error_log = os.path.join(module.tmpdir, "gunicorn.temp.error.log")
    tmp_pid_file = os.path.join(module.tmpdir, "gunicorn.temp.pid")

    # remove temp file if exists
    remove_tmp_file(tmp_pid_file)
    remove_tmp_file(tmp_error_log)

    # obtain app name and venv
    params = module.params
    app = params["app"]
    venv = params["venv"]
    pid = params["pid"]

    # use venv path if exists
    if venv:
        gunicorn_command = f"{venv}/bin/gunicorn"
    else:
        gunicorn_command = module.get_bin_path("gunicorn")

    # to daemonize the process
    options = ["-D"]

    # fill options
    for option in gunicorn_options:
        param = params[option]
        if param:
            options.append(gunicorn_options[option])
            options.append(param)

    error_log = search_existing_config(params["config"], "errorlog")
    if not error_log:
        # place error log somewhere in case of fail
        options.append("--error-logfile")
        options.append(tmp_error_log)

    pid_file = search_existing_config(params["config"], "pid")
    if not params["pid"] and not pid_file:
        pid = tmp_pid_file

    # add option for pid file if not found on config file
    if not pid_file:
        options.append("--pid")
        options.append(pid)

    # put args together
    args = [gunicorn_command] + options + [app]
    rc, out, err = module.run_command(args, use_unsafe_shell=False, encoding=None)

    if not err:
        # wait for gunicorn to dump to log
        time.sleep(0.5)
        if os.path.isfile(pid):
            with open(pid) as f:
                result = f.readline().strip()

            if not params["pid"]:
                os.remove(pid)

            module.exit_json(changed=True, pid=result, debug=" ".join(args))
        else:
            # if user defined own error log, check that
            if error_log:
                error = f"Please check your {error_log.strip()}"
            else:
                if os.path.isfile(tmp_error_log):
                    with open(tmp_error_log) as f:
                        error = f.read()
                    # delete tmp log
                    os.remove(tmp_error_log)
                else:
                    error = "Log not found"

            module.fail_json(msg=f"Failed to start gunicorn. {error}", error=err)

    else:
        module.fail_json(msg=f"Failed to start gunicorn {err}", error=err)


if __name__ == "__main__":
    main()
